from abstract_window_manager import AbstractWindowManager,GUIManager
from abstract_next_read_manager import NextReadManager
from abstract_gui import make_component
window_mgr=AbstractWindowManager()
next_read_mgr=NextReadManager()
next_read_mgr.execute_queue()
window_name = window_mgr.add_window('test',[make_component("text","text window"),make_component("Button","Exit")])
window = window_mgr.get_window_method(window_name=window_name)
enum = window_mgr.get_any_enumeration(window_name=window_name)
if enum==0:
    input(enum)
def event_handler(event, values,window,):
    if event == None:
        print('exiting...')
    return True  # Return True to continue the loop
def get_screen_size():
    return make_component("Window").get_screen_size()
# Define a function to manage the event loop
 
window_mgr.while_window(window_name=window, event_handlers=event_handler)




from abstract_ai import SaveManager,get_any_value,safe_json_loads
files = """C:/Users/jrput/Documents/python projects/Modules/test_modules/test_abstract_test_ai/response_data/2023-11-05/gpt-4-0613/1699214399.json
C:/Users/jrput/Documents/python projects/Modules/test_modules/test_abstract_test_ai/response_data/2023-11-05/gpt-4-0613/1699214352.json
C:/Users/jrput/Documents/python projects/Modules/test_modules/test_abstract_test_ai/response_data/2023-11-05/gpt-4-0613/1699214319.json
C:/Users/jrput/Documents/python projects/Modules/test_modules/test_abstract_test_ai/response_data/2023-11-05/gpt-4-0613/1699214257.json
C:/Users/jrput/Documents/python projects/Modules/test_modules/test_abstract_test_ai/response_data/2023-11-05/gpt-4-0613/1699214198.json
C:/Users/jrput/Documents/python projects/Modules/test_modules/test_abstract_test_ai/response_data/2023-11-05/gpt-4-0613/1699214175.json
C:/Users/jrput/Documents/python projects/Modules/test_modules/test_abstract_test_ai/response_data/2023-11-05/gpt-4-0613/1699213848.json
C:/Users/jrput/Documents/python projects/Modules/test_modules/test_abstract_test_ai/response_data/2023-11-05/gpt-4-0613/1699213714.json
C:/Users/jrput/Documents/python projects/Modules/test_modules/test_abstract_test_ai/response_data/2023-11-05/gpt-4-0613/1699214528.json
C:/Users/jrput/Documents/python projects/Modules/test_modules/test_abstract_test_ai/response_data/2023-11-05/gpt-4-0613/1699214505.json
C:/Users/jrput/Documents/python projects/Modules/test_modules/test_abstract_test_ai/response_data/2023-11-05/gpt-4-0613/1699214471.json
C:/Users/jrput/Documents/python projects/Modules/test_modules/test_abstract_test_ai/response_data/2023-11-05/gpt-4-0613/1699214440.json""".split('\n')
import json
for each in files:
    data = SaveManager.read_saved_json(each)
    content = get_any_value(data['response'],'content')
    if isinstance(content,str):
        content = json.loads(content.replace('False','false').replace("True",'true'))
    if isinstance(content,dict):
        
        input(content['api_response'])



from abstract_gui import AbstractWindowManager
from PromtBuilder import *
from abstract_ai import ModelManager,InstructionManager

prompt_mgr = PromptManager(dist_number=None,
              bot_notation=None,
              generate_title=None,
              chunk_descriptions=None,
              request_chunks=None,
              prompt_as_previous=None,
              token_adjustment=None)

prompt_mgr.create_prompt
calculate_token_distribution(
 role='assistant',
 completion_percentage=40,
 prompt_data=None,
 request=None,
 token_dist=None,
 bot_notation=None,
 chunk=None,
 chunk_type=None)




import re
import json

def preprocess_string(s):
    # Replace \" with a unique marker
    s = s.replace('\\"', '*<d<*')
    # Replace any remaining " with '
    s = s.replace('"', "'")
    # Replace the original single quotes (now all are delimiters) with "
    s = s.replace("'", '"')
    # Replace the unique markers back to \"
    s = s.replace('*<d<*', '\\"')
    return s

def postprocess_string(s):
    # After loading the JSON and processing it, if you need to output the JSON
    # or parts of it as a string with the original quote characters, do the reverse.
    s = s.replace('\\"', '*<d<*')
    s = s.replace('"', "'")
    s = s.replace("*<d<*", '\\"')
    return s

def if_key_in_value(data, key):
    if isinstance(data, dict):
        for k, v in data.items():
            if k == key:
                return True
            if isinstance(v, (dict, list)):
                if if_key_in_value(v, key):
                    return True
    elif isinstance(data, list):
        for item in data:
            if if_key_in_value(item, key):
                return True
    return False

def get_path_to_key(data, key, path=None):
    if path is None:
        path = []
    if isinstance(data, dict):
        for k, v in data.items():
            new_path = path + [k]
            if k == key:
                return new_path
            if isinstance(v, (dict, list)):
                found_path = get_path_to_key(v, key, new_path)
                if found_path:
                    return found_path
    elif isinstance(data, list):
        for idx, item in enumerate(data):
            new_path = path + [idx]
            if isinstance(item, (dict, list)):
                found_path = get_path_to_key(item, key, new_path)
                if found_path:
                    return found_path
    return None

def convert_and_find_key(data, key):
    if not isinstance(data, (dict, list)):
        data = preprocess_string(data)
        try:
            data = json.loads(data)
        except json.JSONDecodeError as e:
            print(f"JSON decoding failed: {e}")
            return None, None

    if if_key_in_value(data, key):
        path = get_path_to_key(data, key)
        return path, get_value_from_path(data, path)
    else:
        return None, None

def get_value_from_path(data, path):
    for key in path:
        data = data[key] if isinstance(data, dict) else data[int(key)]
    return data

# Example Usage
from ResponseBuilder import SaveManager
save_mgr = SaveManager()
output = save_mgr.read_saved_json(file_path="C:/Users/jrput/Documents/python projects/Modules/test_modules/test_abstract_test_ai/response_data/2023-11-05/gpt-4-0613/1699175901.json")['response']
input(str(output).split('api_response')[1])

path, value = convert_and_find_key(output, 'api_response')
if path is not None:
    print(f"Path to '{key_to_find}': {path}")
    print(f"Value at path: {value}")
else:
    print(f"Key '{key_to_find}' not found")



import os
import re
import json
from abstract_utilities.json_utils import get_value_from_path,get_any_value,safe_json_loads
def preprocess_string(s):
    # Replace \" with a unique marker
    s = s.replace('\\"', '*<d<*')
    # Replace any remaining " with '
    s = s.replace('"', "'")
    # Replace the original single quotes (now all are delimiters) with "
    s = s.replace("'", '"')
    # Replace the unique markers back to \"
    s = s.replace('*<d<*', '\\"')
    return s
def postprocess_string(s):
    # After loading the JSON and processing it, if you need to output the JSON
    # or parts of it as a string with the original quote characters, do the reverse.
    s = s.replace('\\"', '*<d<*')
    s = s.replace('"', "'")
    s = s.replace("*<d<*", '\\"')
    return s
def get_any_value_converter(output,key,return_dict=False):
    return_desired_dict=False
    # Preprocess the string to temporarily replace quotes
    if isinstance(output,dict):
        if return_dict:
            return output
        return output[key]
    json_ready_string = preprocess_string(output)
    try:
        # Load the string as a JSON object
        data_json = json.loads(json_ready_string)
        data_json=data_json
        value = get_any_value(data_json,key)
        # Example of processing and then converting back to a string
        output_string = json.dumps(value, indent=4)
        output_string=postprocess_string(output_string)
        return_desired_dict=data_json
    except json.JSONDecodeError as e:
        # Handle JSON errors
        print("Decoding JSON has failed:", e)
    if return_dict and return_desired_dict:
        return return_desired_dict
    return output_string
def find_paths_to_key_in_string(data, key, current_path=None, paths=None):
    if paths is None:
        paths = []
    if current_path is None:
        current_path = []
    if isinstance(data, dict):
        for k, v in data.items():
            new_path = current_path + [k]
            if k == key:
                paths.append(new_path)
            if isinstance(v, (dict, list, str)):
                find_paths_to_key_in_string(v, key, new_path, paths)
    elif isinstance(data, list):
        for i, item in enumerate(data):
            new_path = current_path + [i]
            if isinstance(item, (dict, list, str)):
                find_paths_to_key_in_string(item, key, new_path, paths)
    elif isinstance(data, str):
        # Preprocess string to replace problematic quotes
        processed_string = preprocess_string(data)
        try:
            # Attempt to convert the string to JSON and search within it
            json_data = json.loads(processed_string)
            find_paths_to_key_in_string(json_data, key, current_path, paths)
        except json.JSONDecodeError:
            # If decoding fails, check if the string itself contains the key
            if key in data:
                paths.append(current_path)
    return paths
def get_value_from_object(data=None,key_to_find=None):
    save_mgr = SaveManager()
    dict_values=[]
    if isinstance(data,str):
        if os.path.exists(data):
            data=save_mgr.read_saved_json(file_path=data)
    data=safe_json_loads(data)
    value = get_any_value(data,key_to_find)
    paths = find_paths_to_key_in_string(data,key_to_find)
    for path in paths:
        string_value = get_value_from_path(data,path[:-1])
        dict_values.append(get_any_value_converter(string_value,key_to_find,return_dict=True))
    return dict_values
from ResponseBuilder import SaveManager
input(get_value_from_object("C:/Users/jrput/Documents/python projects/Modules/test_modules/test_abstract_test_ai/response_data/2023-11-05/gpt-4-0613/1699175901.json",'api_response'))
